#include <limits.h>
#include "lir.h"

/*
 * Optimize offset sizes
 *   Reduces instructions which take an offset, such as LDIL, JMPL to shorter
 * forms where appropriate.
 */

static void lir_calc_max_offsets(void);
static void reduce_offset(lir_instr_t *instr);
static int lir_instr_size(lir_instr_t *instr);

//{{{
void lir_opt_offset_size(void)
{
	int ii;
	
	lir_calc_max_offsets();
	
	for(ii=0; ii<instr_num; ii++)
	{
		switch(lir_data[ii].type)
		{
			case LIR_JMPL:
			case LIR_BRAL:
				reduce_offset(&lir_data[ii]);
				break;
			default:
				continue;
		}
	}
}
//}}}
//{{{
static void reduce_offset(lir_instr_t *instr)
{
	int ii;
	long offset;
	
	for(ii=0; ii<instr_num; ii++)
	{
		if(lir_data[ii].type == LIR_LABEL && idcmp(lir_data[ii].args.lbl.identifier, instr->args.target)) {
			offset = lir_data[ii].location - instr->location;
			
			if(offset <= SCHAR_MAX && offset >= SCHAR_MIN)
			{
				switch(instr->type)
				{
					case LIR_JMPL: instr->type = LIR_JMPB; break;
					case LIR_BRAL: instr->type = LIR_BRAB; break;
				}
			} else if(offset <= SHRT_MAX && offset >= SHRT_MIN)
			{
				switch(instr->type)
				{
					case LIR_JMPL: instr->type = LIR_JMPW; break;
					case LIR_BRAL: instr->type = LIR_BRAW; break;
				}
			}
			return;
		}
	}
}
//}}}
//{{{
static void lir_calc_max_offsets(void)
{
	int ii;
	long location = 0;
	
	for(ii=0; ii<instr_num; ii++)
	{
		lir_data[ii].location = location;
		location += lir_instr_size(&lir_data[ii]);
	}
}
//}}}

//{{{
static int lir_instr_size(lir_instr_t *instr)
{
	if(instr->type < num_instructions)
		return instructions[instr->type].size;
	
	switch(instr->type)
	{
		default:
			return 0;
	}
}
//}}}
