%{
#include <stdio.h>
#include "y.tab.h"
#include "mkdat.h"

#ifndef YYSTYPE
#	define YYSTYPE int
#endif
extern YYSTYPE yylval;

%}

%option yylineno

%%
[+\-*/%!\|\^\&\<\>\=:]   { return *yytext;         }
">="                    { return OP_GTEQUAL;      }
"<="                    { return OP_LTEQUAL;      }
"<<"                    { return OP_LEFTSHIFT;    }
">>"                    { return OP_RIGHTSHIFT;   }
"=="                    { return OP_EQEQ;         }
"!="                    { return OP_NOTEQ;        }
"&&"                    { return OP_AND;          }
"||"                    { return OP_OR;           }
";"                     { return TOK_SEMI;        }
"\{"                    { return TOK_LBRACE;      }
"\}"                    { return TOK_RBRACE;      }

"link"                  { return TOK_LINK;        }
"export"                { return TOK_EXPORT;      }
"exporttable"           { return TOK_EXPORTTABLE; }
"divert"                { return TOK_DIVERT;      }
"string"                { return TOK_KEY_STRING;  }
"even"                  { return TOK_EVEN;        }
"const"                 { return TOK_CONST;       }
"ulong"                 { yylval=type_ulong;   return TOK_TYPE; }
"slong"                 { yylval=type_slong;   return TOK_TYPE; }
"uint"                  { yylval=type_uint;    return TOK_TYPE; }
"sint"                  { yylval=type_sint;    return TOK_TYPE; }
"ushort"                { yylval=type_ushort;  return TOK_TYPE; }
"sshort"                { yylval=type_sshort;  return TOK_TYPE; }
"uchar"                 { yylval=type_uchar;   return TOK_TYPE; }
"schar"                 { yylval=type_schar;   return TOK_TYPE; }

[0-9]+                  { yylval=atoi(yytext); return TOK_NUMBER; }
\<(\\>|[^>]+)\>         { yylval=parse_char_constant(yytext); return TOK_NUMBER; }
\(                      { return TOK_LPAREN; }
\)                      { return TOK_RPAREN; }
\"(\\.|[^\\"])*\"       { yylval=string_table_add(yytext); return TOK_STRING; }
[_a-zA-Z][_a-zA-Z0-9]*  { yylval=get_identifier(yytext); return TOK_IDENTIFIER; }
[ \t]+                  /* ignore whitespace */
\n                      /* ignore linefeeds */
\#[^\n]*\n              /* ignore #...-style comments */
\/\/[^\n]*\n            /* ignore //...-style comments */
.                       /* ignore bad characters */
%%

